"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aes_oop_1 = require("aes-oop");
class EncryptedStream {
    constructor(_eventName, _randomized) {
        this.eventName = _eventName;
        this.key = _randomized;
        this.synced = false;
        this.syncFn = null;
        this.listenForSync();
    }
    listenWith(func) {
        document.addEventListener(this.eventName, (event) => {
            if (!this.synced) return false;
            let msg = JSON.parse(event.detail);
            msg = (this.synced && typeof msg === 'string') ? aes_oop_1.default.decrypt(msg, this.key) : msg;
            func(msg);
        });
    }
    send(data, to) {
        const addSender = () => { data.from = this.eventName; };
        const encryptIfSynced = () => { data = (this.synced) ? aes_oop_1.default.encrypt(data, this.key) : data; };
        if (typeof data !== 'object')
            throw new Error("Payloads must be objects");
        addSender();
        encryptIfSynced();
        this.dispatch(JSON.stringify(data), to);
    }
    onSync(fn) {
        this.syncFn = fn;
    }
    sync(to, handshake) {
        this.send({ type: 'sync', handshake }, to);
    }
    listenForSync() {
        document.addEventListener(this.eventName, (event) => {
            let msg = JSON.parse(event.detail);
            if(!msg.hasOwnProperty('type')) return false;
            if(msg.type === 'sync') this.ackSync(msg);
            if(msg.type === 'synced') this.synced = true;
        });
    }
    ackSync(msg) {
        this.send({ type: 'synced' }, msg.from);
        this.key = msg.handshake;
        this.synced = true;
        this.syncFn();
    }
    dispatch(encryptedData, to) { document.dispatchEvent(this.getEvent(encryptedData, to)); }
    getEvent(encryptedData, to) { return new CustomEvent(to, this.getEventInit(encryptedData)); }
    getEventInit(encryptedData) { return { detail: encryptedData }; }
}
exports.EncryptedStream = EncryptedStream;
exports.default = EncryptedStream;
//# sourceMappingURL=EncryptedStream.js.map