import Plugin from '../Plugin'
import { ProtocolEnum } from '../../models/Enum'
import Network from '../../models/Network'
import EncryptHelper from '../../utils/EncryptHelper'
import RandomFactory from '../../utils/RandomFactory'
import { kDMC } from '../../constKeys/ConstKeys'
import { chain_name, chain_protocol, chain_host, chain_port, chain_chainId, create_tunnel, public_key_prefix, create_key } from '../../config'
import { JSEncrypt } from 'jsencrypt'
import cryptojs from 'crypto-js'

import { getKeys } from '../../utils/getKeys'


export default class DMC extends Plugin {
    private tunnel: string = create_tunnel

    constructor() {
        super(kDMC, public_key_prefix)
    }

    public getEndorsedNetwork(): Network {
        return new Network({
            name: chain_name,
            protocol: chain_protocol,
            host: chain_host,
            port: chain_port,
            blockchain: this.name,
            chainId: chain_chainId
        })
    }

    /**
     * 系统创建账户
     * @param name 账户名
     * @param publicKey 公钥
     */
    public async createAccountBySystem(name: string, publicKey: string): Promise<boolean> {
        const timestamp = new Date().toISOString()
        const encrypt = new JSEncrypt()
        encrypt.setPrivateKey(await getKeys())
        // encrypt.setPrivateKey(await getKeys())
        // encrypt.setPrivateKey(cryptojs.AES.decrypt(await getKeys() as any, 'key666').toString())
        const data = JSON.stringify({
            account: name,
            public_key: publicKey,
            timestamp,
        })

        // const t = +new Date()
        // const str = `${name}${publicKey}${create_key}${t}`
        // const hash = EncryptHelper.md5(str)
        return new Promise((resolve, reject) => {   
            fetch(this.tunnel, {
                method: 'POST',
                body: JSON.stringify({
                    account: name,
                    public_key: publicKey,
                    timestamp,
                    sign: encrypt.sign(data, cryptojs.SHA256 as any, 'sha256'),
                    // pubkey: publicKey,
                    // hash,
                    // t,
                }),
                headers: {
                    'Content-Type': 'application/json',
                },
            })
                .then((r) => r.json())
                .then((json) => {
                    if (!json) reject(json)
                    else if (json.message === 'success') resolve(true)
                    else reject(json)
                })
                .catch((e) => reject(e))
        })
    }
}