import React from 'react'
import ReactDOM from 'react-dom'

import apis from './utils/ApiHelper'
import PromptMain from './views/prompt/prompt'
import Prompt from './models/Prompt';
import DMCScatter from './models/DMCScatter';
import Keychain from './models/Keychain';
import Keypair from './models/Keypair'
import Identity from './models/Identity'
import Permission from './models/Permission'
import Signature from './models/Signature'
import * as ConstKeys from './constKeys/ConstKeys'

class PromptWindow {
    constructor() {
        let promptData;

        chrome.runtime.sendMessage(ConstKeys.INIT_PROMPT)

        chrome.runtime.onConnect.addListener(port => {
            port.onMessage.addListener(message => {
                if(port.name === 'popup' && message.type === ConstKeys.PROMPT_DATA){
                    const { type, domain, network, data, scatter:{keyChain: {keypairs, identitys, permissions, signatures}, setting, hash} } = message.payload
                    keypairs.map((keypair:any) => {
                        return new Keypair(keypair)
                    })
                    identitys.map((identity:any) => {
                        return new Identity(identity)
                    })
                    permissions.map((permission:any) => {
                        return new Permission(permission)
                    })
                    signatures.map((signature:any) => {
                        return new Signature(signature)
                    })
            
                    const keyChain = new Keychain(keypairs, identitys, permissions, signatures);

                    promptData = new Prompt(
                        type,
                        domain,
                        network,
                        data,
                        port,
                        new DMCScatter(keyChain, setting, hash)
                    );
        
                    const prompt = promptData || apis.runtime.getBackgroundPage().notification || null
        
                    ReactDOM.render(
                        <PromptMain prompt={prompt} />,
                        document.getElementById('prompt') as HTMLElement
                    );
                }
            })
        })
    }
}

new PromptWindow()