import axios from 'axios'
import { tracker_url, chain_host, chain_port } from '../config'

function baseGraphQLRequest(params: any, urlparams?: any, responseType?: any) {
    return axios({
        method: 'POST',
        url: `${tracker_url}/2.1`,
        withCredentials: true,
        headers: {
            'Content-Type': 'application/graphql',
        },
        data: params,
        params: urlparams,
        responseType,
    }).catch((e) => {
        return Promise.resolve(e.response)
    })
}

export function findRecord(name?: string, pagination?: any) {
    const params = `{
    find_tokens_action(
            skip: 0,
            limit: 1000,
            order: "-createdAt,id",
            where: {
                or: [{
                    and:[{
                        or:[
                             {
                               account_from_id: "${name}"
                             },
                             {
                               account_to_id: "${name}"
                             }
                        ]},
                        {
                          contract_action:{
                              in:[
                                "dmc.token/transfer",
                                "dmc.token/extransfer",
                              ]
                        }
                        }
                    ]
                },
                ]

            },
    ){
        account_from{
          id
        }
        account_to{
          id
        }
        contract_action
        action{
          rawData
        }
        id
        createdAt
      }
  }
  `
    return baseGraphQLRequest(params)
}

export function findLockRecord(name?: string, pagination?: any) {
    const params = `{
    find_tokens_action(
            skip: 0,
            limit: 1000,
            order: "-createdAt,id",
            where: {
                or: [{
                    and:[{
                        or:[
                             {
                               account_from_id: "${name}"
                             },
                             {
                               account_to_id: "${name}"
                             }
                        ]},
                        {
                          contract_action:{
                              in:[
                                "dmc.token/exunlock",
                                "dmc.token/exlock"
                              ]
                        }
                        }
                    ],
                },
                {
                    and:[{
                        or:[
                             {
                               account_to_id: "${name}"
                             }
                        ]},
                        {
                          contract_action:{
                              in:[
                                "dmc.token/redeemrec",
                                "dmc.token/exlocktrans",
                              ]
                        }
                        }
                    ]
                }
                ]

            },
    ){
        account_from{
          id
        }
        account_to{
          id
        }
        contract_action
        action{
          rawData
        }
        id
        createdAt
      }
  }
  `
    return baseGraphQLRequest(params)
}

export const getVersion = () => {
    return axios({
        method: 'get',
        url: `http://${chain_host}:${chain_port}/extensionVersion.json?v=${Date.now()}`,
        withCredentials: true,
    })
}
