import Prompt from '../models/Prompt'
import apis from './ApiHelper'
import Identity from '../models/Identity'
import * as ConstKeys from '../constKeys/ConstKeys'
import { LocalStream } from 'extension-streams'

class NotificationHelper {
    private readonly target: string = 'ScatterPrompt'
    private openedWindow: Window = null

    private destory(): void {
        if (this.openedWindow) {
            // @ts-ignore
            chrome.windows.remove(this.openedWindow.id)
        }
    }

    private async popUp(notification: Prompt) {
        try {
            const {width:winW, height:winH} = await chrome.windows.getCurrent()
            const w = (600 / 1920 * winW) << 0
            const h = (400 / 1080 * winH) << 0

            const topX = parseInt(`${(winH - h) / 2}`)
            const leftY = parseInt(`${(winW - w) / 2}`)
            // const url = `${apis.runtime.getURL('/prompt.html')}?data=${encodeURIComponent(JSON.stringify(notification))}`
            const url = `${apis.runtime.getURL('/prompt.html')}`

            LocalStream.watch((message: any) => {

                if(message === ConstKeys.INIT_PROMPT){
                    chrome.tabs.query({active: true, currentWindow: true}, (tabs) => {
                        
                        const port = chrome.tabs.connect(tabs[0].id, {
                            name: "popup",
                        });

                        port.postMessage({ type: ConstKeys.PROMPT_DATA, payload: notification })

                        port.onMessage.addListener(function(msg){
                            // Todo: IdentityHelper.getOrRequestIdentity 作为 callback 接收identity, fromPermission两个参数，目前还没发现哪里传输 fromPermission 参数。
                            // notification.responder(new Identity(msg.publicKey, msg.name, msg.accounts))
                            if (msg) {
                                notification.responder(new Identity(msg.publicKey, msg.name, msg.accounts, msg.type))
                            }
                        })
                    })
                }

            })

            // let win: any = window.open(url, this.target, `width=${w},height=${h},resizable=0,top=${topX},left=${leftY},titlebar=0`)
            let win: any = await chrome.windows.create({focused: true, url, type: 'popup', width: w, height: h, top: topX, left: leftY})
            win.data = notification
            this.openedWindow = win
            return this.openedWindow
        } catch (e) {
            return null
        }
    }

    /**
     * 打开window
     * @param notification 
     */
    public async openWindow(notification: Prompt) {
        this.destory()
        this.popUp(notification)
    }
}

export default new NotificationHelper()