import Toast from '../views/component/toast/index'
import { kToastDurtion } from '../config'
import LocalizationHelper from '../localization/LocalizationHelper'

export default function reformChainError(
    error: any,
    name?: any,
    callback?: any
) {
    let errorObject = error
    if (typeof errorObject === 'string') {
        errorObject = JSON.parse(errorObject).error
    }
    if (errorObject.toString() === 'TypeError: Failed to fetch') {
        Toast.fail(LocalizationHelper.valueForKey('net_error'), kToastDurtion)
        return
    }
    if (errorObject.message === 'wrong password') {
        // 密码错误
        Toast.info(
            LocalizationHelper.valueForKey('password_error'),
            kToastDurtion
        )
    } else if (errorObject.message !== 'user reject') {
        // 用户取消,不做处理
        if (
            errorObject.name === 'expired_tx_exception' &&
            errorObject.what === 'Expired Transaction'
        ) {
            // 交易超时
            Toast.info(
                LocalizationHelper.valueForKey('request_timeout'),
                kToastDurtion
            )
        } else if (
            errorObject.name === 'ram_usage_exceeded' &&
            errorObject.what === 'Account using more than allotted RAM usage'
        ) {
            Toast.info(
                LocalizationHelper.valueForKey('ram_insuffient'),
                kToastDurtion
            )
        } else if (errorObject.details && errorObject.details.length > 0) {
            if (
                errorObject.details[0].message ===
                'assertion failure with message: must purchase a positive amount'
            ) {
                Toast.info(
                    LocalizationHelper.valueForKey('ram_too_low'),
                    kToastDurtion
                )
            }  else if (
                errorObject.details[0].message ===
                'assertion failure with message: memo has more than 256 bytes'
            ) {
                Toast.info(
                    LocalizationHelper.valueForKey('memo_more_than_256'),
                    kToastDurtion
                )
            } else if (
                errorObject.details[0].message ===
                `transaction declares authority '{"actor":"${name}","permission":"active"}', but does not have signatures for it.`
            ) {
                Toast.info(
                    LocalizationHelper.valueForKey('authority_error'),
                    kToastDurtion
                )
            } else if (
                errorObject.details[0].message ===
                'assertion failure with message: cannot transfer to self'
            ) {
                Toast.info(
                    LocalizationHelper.valueForKey('receiver_not_self'),
                    kToastDurtion
                )
            } else if (
                errorObject.details[0].message ===
                'assertion failure with message: no balance object found.'
            ) {
                Toast.info(
                    LocalizationHelper.valueForKey('NO_ENOUGH_DMC'),
                    kToastDurtion
                )
            } else if (
                errorObject.details[0].message ===
                    'assertion failure with message: no balance object found.' ||
                errorObject.details[0].message ===
                    'assertion failure with message: overdrawn balance when sub balance'
            ) {
                Toast.info(
                    LocalizationHelper.valueForKey('balance_insuffient'),
                    kToastDurtion
                )
            } else if (
                errorObject.details[0].message ===
                    'assertion failure with message: to account does not exist' ||
                errorObject.details[0].message ===
                    'assertion failure with message: cannot transfer lock tokens to nonexist account' ||
                errorObject.details[0].message ===
                    'assertion failure with message: to account no exists!' ||
                errorObject.details[0].message ===
                    `unknown key (boost::tuples::tuple<bool, eosio::chain::name, boost::tuples::null_type, boost::tuples::null_type, boost::tuples::null_type, boost::tuples::null_type, boost::tuples::null_type, boost::tuples::null_type, boost::tuples::null_type, boost::tuples::null_type>): (0 ${name}): pending console output: `
            ) {
                Toast.info(
                    LocalizationHelper.valueForKey('receiver_not_exist'),
                    kToastDurtion
                )
            }  else if (
                errorObject.what ===
                'Provided keys, permissions, and delays do not satisfy declared authorizations'
            ) {
                Toast.info(
                    LocalizationHelper.valueForKey('authority_error'),
                    kToastDurtion
                )
            } else if (
                errorObject.message ===
                `unknown key (boost::tuples::tuple<bool, eosio::chain::name, boost::tuples::null_type, boost::tuples::null_type, boost::tuples::null_type, boost::tuples::null_type, boost::tuples::null_type, boost::tuples::null_type, boost::tuples::null_type, boost::tuples::null_type>): (0 ${name})`
            ) {
                // 该账户不存在
                if (callback) {
                    callback({ isExist: false })
                }
            } else {
                Toast.info(`${errorObject.details[0].message}`, kToastDurtion)
            }
        } else {
            try {
                Toast.info(`${errorObject.toString()}`, kToastDurtion)
            } catch (err) {
                Toast.info(
                    `'${LocalizationHelper.valueForKey(
                        'unknow_error'
                    )}: '${err.toString()}`,
                    kToastDurtion
                )
            }
        }
    }
    if (callback) {
        callback()
    }
}
