import React, { Component } from 'react'
import './index.less'
import Toast from '../../component/toast/index'
import EncryptHelper from '../../../utils/EncryptHelper'
import LocalizationHelper from '../../../localization/LocalizationHelper'
import RandomFactory from '../../../utils/RandomFactory'
import { kPasswordLen, kToastDurtion } from '../../../config'
import { IStoreState } from '../../../reducer'
import { connect } from 'react-redux'

interface IPopParams {
    y: number
}

interface IPopProps {
    ref?: (e: ConfirmModal) => void
    callback?: () => void
    password?: string
}

interface IPopState {
    visible: boolean
    password: string
}

interface IOption {
    icon?: any
    title: string
}

/**
 * 非受控组件
 */
export default class ConfirmModal extends Component<IPopProps, IPopState> {
    private timer: number
    private r: number
    private y: number
    private w: number = 110
    private hash: any
    private options: Array<IOption>

    constructor(props: IPopProps) {
        super(props)
    }
    state = {
        visible: false,
        password: '',
    }

    public show(params: IPopParams) {
        this.y = params.y
        this.setState({ visible: true })
    }

    public hide() {
        this.setState({ visible: false })
    }

    private itemClick() {
        const { password } = this.state
        const { callback } = this.props
        if (!password) {
            return
        }
        const pass = this.checkPwd(password)
        if (!pass) {
            return
        }
        callback()
        this.hide()
    }

    checkPwd(pwd: string): boolean {
        const { password } = this.props
        if (pwd.length <= 0) {
            Toast.info(
                LocalizationHelper.valueForKey('password_empty'),
                kToastDurtion
            )
            return false
        }
        const hashed = EncryptHelper.md5(
            EncryptHelper.md5(pwd),
            RandomFactory.systemSalt()
        )
        if (hashed !== password) {
            Toast.info(
                LocalizationHelper.valueForKey('password_error'),
                kToastDurtion
            )
            return false
        }
        return true
    }

    render() {
        const { visible } = this.state
        if (!visible) {
            return null
        }
        const { r, y, options } = this
        const style: any = { top: y, right: r }
        return (
            <div
                id='pop_mask'
                className={visible ? 'shadow' : 'none'}
                style={{ height: 'calc(100vh + y)' }}
            >
                <div
                    className='confirm_modal'
                    style={
                        visible ? { ...style } : { display: 'none', ...style }
                    }
                    // onClick={() => this.hide()}
                >
                    <div className='title'>
                        {LocalizationHelper.valueForKey(
                            'confirm_input_password'
                        )}
                    </div>
                    <div className='password'>
                        <input
                            className='textarea scroll input'
                            type='password'
                            onChange={(e) => {
                                this.setState({
                                    password: e.target.value,
                                })
                            }}
                        ></input>
                    </div>
                    <div className='btn'>
                        <button className='cancel' onClick={() => this.hide()}>
                            {LocalizationHelper.valueForKey('cancel')}
                        </button>
                        <button
                            className='confirm'
                            onClick={() => this.itemClick()}
                        >
                            {LocalizationHelper.valueForKey('determine')}
                        </button>
                    </div>
                </div>
            </div>
        )
    }
}
