import React, { Component } from 'react'
import { bindActionCreators, Dispatch } from 'redux'
import { format } from 'date-fns'
import { match } from 'react-router-dom'
import { History } from 'history'
import { connect } from 'react-redux'
import { IStoreState } from '../../reducer/index'
import Identity from '../../models/Identity'
import Permission from '../../models/Permission'
import * as KeyChainActions from '../../actions/keychain'
import LocalizationHelper from '../../localization/LocalizationHelper'
import Father from '../component/father'
import DMCScatterInterface from '../../interface/Interface'
import Modal from '../component/modal/index'
import UtilsHelper from '../../utils/UtilsHelper'
import GroupHelper from '../../utils/GroupHelper'
import ReformTokenName from '../../utils/ReformTokenName'
import { findRecord } from '../../service/request'
import { account, lockAccount } from '../../service/action'
import BigNumber from 'bignumber.js'
import Toast from '../component/toast/index'
import reformChainError from '../../utils/reformError'

interface IAccountManagerProps {
    identitys: Array<Identity>
    match?: match
    permissions: Array<Permission>
    history?: any
    dispatch?: Dispatch
}

interface IHashProps {
    hash: string
}

class Assets extends Component<IAccountManagerProps> {
    private keyChainAction: typeof KeyChainActions

    constructor(props: IAccountManagerProps) {
        super(props)
        this.keyChainAction = bindActionCreators(
            KeyChainActions,
            props.dispatch
        )
    }

    componentDidMount(): void {
        Toast.loading('', 0)
        const { history } = this.props
        const name = history?.location?.state?.name
        this.setState({ name: name })
        ;(async () => {
            try {
                const data = await findRecord(name)
                this.setState({
                    record_list: data?.data?.data?.find_tokens_action,
                })
                const asset_data = await account(name)
                const dmc_asset = asset_data?.rows?.filter((item: any) => {
                    return item?.balance?.quantity?.split(' ')[1] === 'DMC'
                })[0]?.balance?.quantity
                this.setState({ dmc_quantity: dmc_asset || '0.0000' })
                const lock_data = await lockAccount(name)
                const lock_asset = lock_data?.rows?.filter((item: any) => {
                    return item?.balance?.quantity?.split(' ')[1] === 'DMC'
                })
                const dmcSum = lock_asset.reduce((pre: any, cur: any) => {
                    return new BigNumber(pre)
                        .plus(new BigNumber(cur.balance.quantity.split(' ')[0]))
                        .toFixed(4)
                }, 0)
                this.setState({ dmc_locked: new BigNumber(dmcSum).toFixed(4) })
                Toast.hide()
            } catch (error) {
                reformChainError(error)
            }
        })()
    }

    state = {
        showWhichTab: 'assets',
        record_list: [] as any,
        name: '',
        dmc_quantity: '',
        dmc_locked: '',
    }

    render() {
        const { name, dmc_quantity } = this.state
        const { history, match } = this.props
        return (
            <Father
                history={history}
                left={require('../../assets/back.png')}
                title={LocalizationHelper.valueForKey('assets')}
            >
                <div className='assetsTab'>
                    <div>{name}</div>
                    <div className='assetsAmount'>{dmc_quantity}</div>
                    <div className='assetsAction'>
                        {/* <div>
                            <img
                                width={14}
                                src={require('../../assets/receive.png')}
                            ></img>
                            <span
                                onClick={() => {
                                    history.push('/receive', {
                                        present_account: name,
                                    })
                                }}
                            >
                                {LocalizationHelper.valueForKey(
                                    'currency_collection'
                                )}
                            </span>
                        </div>
                        <div>|</div> */}
                        <div>
                            <img
                                width={14}
                                src={require('../../assets/transfer.png')}
                            ></img>
                            <span
                                onClick={() => {
                                    const { history, match } = this.props
                                    const { hash } = match.params as IHashProps
                                    history.push(`/extransfer/${hash}`, {
                                        dmc_asset: dmc_quantity,
                                        present_account: name,
                                    })
                                }}
                            >
                                {LocalizationHelper.valueForKey('transfer')}
                            </span>
                        </div>
                    </div>
                </div>
                <div className='assetsList'>
                    <div className='tabs'>
                        <span
                            className={
                                this.state.showWhichTab === 'assets'
                                    ? `showTab`
                                    : 'tab'
                            }
                            onClick={() => {
                                this.setState({ showWhichTab: 'assets' })
                            }}
                        >
                            {LocalizationHelper.valueForKey('assets')}
                        </span>
                        <span
                            className={
                                this.state.showWhichTab === 'record'
                                    ? `showTab`
                                    : 'tab'
                            }
                            onClick={() => {
                                this.setState({ showWhichTab: 'record' })
                            }}
                        >
                            {LocalizationHelper.valueForKey('assets_record')}
                        </span>
                    </div>
                    <div>
                        {this.state.showWhichTab === 'assets' ? (
                            <div className='assetsContainer'>
                                <div>
                                    <img
                                        width={30}
                                        src={require('../../assets/dmc_pic.png')}
                                    ></img>
                                </div>
                                <div className='amountContainer'>
                                    <div className='totalAmount'>
                                        <div className='key'>DMC</div>
                                        <div className='value'>
                                            {Number(
                                                Number(
                                                    dmc_quantity?.split(' ')[0]
                                                ) +
                                                    Number(
                                                        this.state.dmc_locked
                                                    )
                                            ).toFixed(4)}
                                        </div>
                                    </div>
                                    <div className='availAmount'>
                                        <div className='key'>
                                            {LocalizationHelper.valueForKey(
                                                'available_amount'
                                            )}
                                        </div>
                                        <div className='value'>
                                            {dmc_quantity.split(' ')[0]}
                                        </div>
                                    </div>
                                    <div className='lockAmount'>
                                        <div className='key'>
                                            {LocalizationHelper.valueForKey(
                                                'locked_amount'
                                            )}
                                        </div>
                                        <div
                                            className='value'
                                            style={{ cursor: 'pointer' }}
                                            onClick={() => {
                                                const { hash } =
                                                    match.params as IHashProps
                                                history.push('/lock', {
                                                    present_account: name,
                                                    locked_quantity:
                                                        this.state.dmc_locked,
                                                    hash: hash,
                                                })
                                            }}
                                        >
                                            {Number(
                                                this.state.dmc_locked
                                            ).toFixed(4)}
                                            <span style={{ marginLeft: 10 }}>
                                                {'>'}
                                            </span>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        ) : (
                            this.state?.record_list?.map((item: any) => {
                                const data = item?.action?.rawData?.act?.data
                                return (
                                    <div className='record_detail'>
                                        <div className='top'>
                                            <div className='title'>
                                                {name === data?.from
                                                    ? LocalizationHelper.valueForKey(
                                                          'transfer'
                                                      )
                                                    : LocalizationHelper.valueForKey(
                                                          'currency_collection'
                                                      )}
                                            </div>
                                            <div
                                                className={`amount ${
                                                    name === data?.from
                                                        ? 'to'
                                                        : 'from'
                                                }`}
                                            >
                                                {`${
                                                    name === data?.from
                                                        ? '-'
                                                        : '+'
                                                }${
                                                    data?.quantity?.quantity?.split(
                                                        ' '
                                                    )[0] ||
                                                    data?.quantity?.split(
                                                        ' '
                                                    )[0]
                                                }`}
                                            </div>
                                        </div>
                                        <div className='info'>
                                            <div>
                                                {`${
                                                    name === data?.from
                                                        ? LocalizationHelper.valueForKey(
                                                              'transfer_to'
                                                          )
                                                        : LocalizationHelper.valueForKey(
                                                              'transfer_from'
                                                          )
                                                } ${
                                                    name === data?.from
                                                        ? data.to
                                                        : data.from
                                                }`}
                                            </div>
                                            <div title={data.memo}>
                                                {LocalizationHelper.valueForKey(
                                                    'transfer_memo'
                                                ) +
                                                    ':' +
                                                    ' ' +
                                                    data.memo}
                                            </div>
                                        </div>
                                        <div className='bottom'>
                                            <div className='time'>
                                                {format(
                                                    new Date(
                                                        `${item?.action?.rawData?.block_time}Z`
                                                    ),
                                                    'yyyy-MM-dd HH:mm:ss'
                                                )}
                                            </div>
                                            <div className='status'>
                                                {LocalizationHelper.valueForKey(
                                                    'confirmed'
                                                )}
                                            </div>
                                        </div>
                                    </div>
                                )
                            })
                        )}
                    </div>
                </div>
            </Father>
        )
    }
}

function mapStateToProps({ keychain }: IStoreState) {
    return {
        identitys: keychain.identitys,
        permissions: keychain.permissions,
    }
}

export default connect(mapStateToProps)(Assets)
