import React, { Component } from 'react'
import { bindActionCreators, Dispatch } from 'redux'
import { connect } from 'react-redux'
import { History } from 'history'
import Father from '../component/father'
import LocalizationHelper from '../../localization/LocalizationHelper'
import Pop from '../component/pop'
import Network from '../../models/Network'
import { IStoreState } from '../../reducer'
import * as CreateActions from '../../actions/create'
import {
    kAccountNameRegex,
    kToastDurtion,
    requiredByte,
    requiredCpu,
    requiredNet,
} from '../../config/index'
import Toast from '../component/toast/index'
import PluginFactory from '../../plugins/PluginFactory'
import Identity from '../../models/Identity'
import reformChainError from '../../utils/reformError'

interface ICreateProps {
    history?: History
    networks: Array<Network>
    dispatch?: Dispatch
    name: string
    network: number
    identitys: Array<Identity>
    created: boolean
    privateKey: string
    publicKey?: string
    selected_name?: number
}

class Create extends Component<ICreateProps> {
    private pop: Pop
    private createAction: typeof CreateActions

    constructor(props: ICreateProps) {
        super(props)
        this.createAction = bindActionCreators(CreateActions, props.dispatch)
    }
    state = {
        fromAccount: '',
        publicKey: '',
        privateKey: '',
        hash: '',
        create_name: '',
    }

    componentDidMount() {
        const { created, network, networks, selected_name, identitys } =
            this.props
        if (created) {
            ;(async () => {
                this.setState({
                    fromAccount: identitys[selected_name]?.name?.split('@')[0],
                    hash: identitys[selected_name].hash,
                })
                this.createAction.updateKV(
                    'privateKey',
                    LocalizationHelper.valueForKey('generate_privatekey_ing')
                )
                const plugin = PluginFactory.basePlugin()
                const privateKey = await plugin.randomPrivateKey()
                const network_ = networks[network]
                const plugin2 = await PluginFactory.plugin(network_.blockchain)
                const publicKey = plugin2.privateToPublic(privateKey)
                this.setState({ publicKey: publicKey })
                this.setState({ privateKey: privateKey })
                this.createAction.updateKV('privateKey', privateKey)
            })()
            return
        }
        this.createAction.clear()
    }

    getOptions(): Array<string> {
        const { networks } = this.props
        return networks.map((network) => network.name)
    }

    getAccountOptions(): Array<string> {
        const { identitys } = this.props
        return [
            ...new Set(
                identitys
                    .filter((item) => item.name.split('@')[1] === 'active')
                    .map((id) => id.name.split('@')[0])
            ),
        ]
    }

    async freeCreateDidClick() {
        const { name, history, network, networks } = this.props
        if (!kAccountNameRegex.test(name)) {
            Toast.info(
                LocalizationHelper.valueForKey('account_name_not_match'),
                kToastDurtion
            )
            return
        }
        Toast.loading(LocalizationHelper.valueForKey('check_name_ing'), 0)
        const network_ = networks[network]
        const plugin = PluginFactory.basePlugin()
        const exist = await plugin.checkAccountExist(`${name}`, network_)
        Toast.hide()
        if (typeof exist === 'object') {
            reformChainError(exist)
            return
        }
        if (exist) {
            Toast.info(
                LocalizationHelper.valueForKey('name_exist'),
                kToastDurtion
            )
            return
        }
        history.push('/backup', { status: 'freeToCreate' })
    }

    async costToCreateDidClick() {
        const { privateKey, publicKey, fromAccount, hash, create_name } =
            this.state
        const { name, history, network, networks } = this.props
        if (!kAccountNameRegex.test(create_name)) {
            Toast.info(
                LocalizationHelper.valueForKey('account_name_not_match'),
                kToastDurtion
            )
            return
        }
        Toast.loading(LocalizationHelper.valueForKey('check_name_ing'), 0)
        const network_ = networks[network]
        const plugin = PluginFactory.basePlugin()
        const exist = await plugin.checkAccountExist(`${create_name}`, network_)
        Toast.hide()
        if (typeof exist === 'object') {
            reformChainError(exist)
            return
        }
        if (exist) {
            Toast.info(
                LocalizationHelper.valueForKey('name_exist'),
                kToastDurtion
            )
            return
        }
        history.push('/backup', {
            status: 'costToCreate',
            create_name: create_name,
            privateKey: privateKey,
            publicKey: publicKey,
            fromAccount: fromAccount,
            hash: hash,
        })
    }

    mouseEnter(evt: React.MouseEvent) {
        const { created } = this.props
        const target = evt.target as HTMLElement
        const isChild = evt.target !== evt.currentTarget
        const offsetY = isChild
            ? target.parentElement.parentElement.parentElement.parentElement
                  .scrollTop
            : target.parentElement.parentElement.parentElement.scrollTop
        let y: number = isChild
            ? target.parentElement.offsetTop
            : target.offsetTop
        this.pop.show({
            r: 28,
            y: y + 44 - offsetY,
            w: window.innerWidth - 24 * 2 - 4,
            height: 300,
            options: created ? this.getAccountOptions() : this.getOptions(),
        })
    }

    onSelect(idx: number) {
        const { network } = this.props
        if (network === idx) {
            return
        }
        this.createAction.updateKV('network', idx)
    }

    onAccountSelect(idx: number, hash: any, opt: any) {
        const { network, identitys, selected_name } = this.props
        // if (selected_name === idx) {
        //     return
        // }
        const selected = identitys
            .filter((identity) => {
                return identity.name.split('@')[0] === opt
            })
            .filter((identity) => {
                return identity.name.split('@')[1] === 'active'
            })
        this.createAction.updateKV('selected_name', idx)
        this.setState({ fromAccount: opt?.split('@')[0] })
        this.setState({ hash: selected[0].hash })
    }

    renderCostToCreate() {
        const { publicKey, create_name } = this.state
        const { name, network, selected_name } = this.props
        return (
            <div className='verify costCreate'>
                <span className='input_tips' style={{ marginTop: 12 }}>
                    {LocalizationHelper.valueForKey('cost_creatd_tip')}
                </span>
                <span className='tag'>
                    {LocalizationHelper.valueForKey('paid_account')}
                </span>
                <div className='account_item'>
                    <span className='account_name'>
                        {this.getAccountOptions()[selected_name]}
                    </span>
                    <span
                        className='account_arrow_mask'
                        onClick={(evt) => this.mouseEnter(evt)}
                    >
                        <img
                            className='account_arrow'
                            src={require('../../assets/arrow.png')}
                        />
                    </span>
                </div>
                <span className='tag'>
                    {LocalizationHelper.valueForKey('create_acc_info')}
                </span>
                <div className='create_name'>
                    <span>{LocalizationHelper.valueForKey('create_name')}</span>
                    <input
                        title={LocalizationHelper.valueForKey(
                            'enter_your_name'
                        )}
                        className='textarea scroll input'
                        placeholder={LocalizationHelper.valueForKey(
                            'enter_your_name'
                        )}
                        maxLength={12}
                        value={create_name}
                        onChange={(e) =>
                            this.setState({
                                create_name: e.target.value.trim(),
                            })
                        }
                    />
                </div>
                <div className='pub_key'>
                    <span>{LocalizationHelper.valueForKey('pub_key')}</span>
                    <div className='textarea scroll pk'>{publicKey}</div>
                </div>
                <div className='distribute_ram'>
                    <span>
                        {LocalizationHelper.valueForKey('distribute_ram') +
                            '(byte)'}
                    </span>
                    <input
                        className='textarea scroll input'
                        value={requiredByte}
                        disabled
                    />
                </div>
                <div className='cpu_pledge'>
                    <span>
                        {LocalizationHelper.valueForKey('cpu_pledge') + '(dmc)'}
                    </span>
                    <input
                        className='textarea scroll input'
                        value={requiredCpu}
                        disabled
                    />
                </div>
                <div className='net_pledge'>
                    <span>
                        {LocalizationHelper.valueForKey('net_pledge') + '(dmc)'}
                    </span>
                    <input
                        className='textarea scroll input'
                        value={requiredNet}
                        disabled
                    />
                </div>
                <div
                    className='btn'
                    onClick={() => this.costToCreateDidClick()}
                >
                    {LocalizationHelper.valueForKey('create')}
                </div>
            </div>
        )
    }

    renderFreeToCreate() {
        const { name, network } = this.props
        return (
            <div className='verify'>
                <span className='tag'>
                    {LocalizationHelper.valueForKey('pick_chain')}
                </span>
                <div className='account_item'>
                    <span className='account_name'>
                        {this.getOptions()[network]}
                    </span>
                    <span
                        className='account_arrow_mask'
                        onClick={(evt) => this.mouseEnter(evt)}
                    >
                        <img
                            className='account_arrow'
                            src={require('../../assets/arrow.png')}
                        />
                    </span>
                </div>
                <span className='tag'>
                    {LocalizationHelper.valueForKey('enter_your_name')}
                </span>
                <input
                    className='textarea scroll input'
                    title={LocalizationHelper.valueForKey('enter_your_name')}
                    placeholder={LocalizationHelper.valueForKey(
                        'enter_your_name'
                    )}
                    maxLength={12}
                    value={name}
                    onChange={(e) =>
                        this.createAction.updateKV(
                            'name',
                            e.target.value.trim()
                        )
                    }
                />
                <span className='input_tips'>
                    {LocalizationHelper.valueForKey('name_holder')}
                </span>
                <div className='btn' onClick={() => this.freeCreateDidClick()}>
                    {LocalizationHelper.valueForKey('create')}
                </div>
            </div>
        )
    }

    render() {
        const { identitys, history, created } = this.props
        return (
            <Father
                history={history}
                left={require('../../assets/back.png')}
                title={
                    created
                        ? LocalizationHelper.valueForKey('cost')
                        : LocalizationHelper.valueForKey('create_accout')
                }
            >
                {created
                    ? this.renderCostToCreate()
                    : this.renderFreeToCreate()}
                <Pop
                    ref={(e) => (this.pop = e)}
                    callback={(idx, hash, opt) =>
                        created
                            ? this.onAccountSelect(idx, hash, opt)
                            : this.onSelect(idx)
                    }
                />
            </Father>
        )
    }
}

function mapStateToProps({ setting, create, keychain }: IStoreState) {
    return {
        identitys: keychain.identitys,
        networks: setting.networks,
        network: create.network,
        selected_name: create.selected_name,
        name: create.name,
        created: setting.created,
    }
}

export default connect(mapStateToProps)(Create)
