import React, { Component } from 'react'
import { connect } from 'react-redux'
import { bindActionCreators, Dispatch } from 'redux'
import { match } from 'react-router-dom'
import Father from '../component/father'
import { History } from 'history'
import LocalizationHelper from '../../localization/LocalizationHelper'
import { IStoreState } from '../../reducer'
import * as SettingActions from '../../actions/setting'
import { LanguageEnum } from '../../models/Enum'
import Toast from '../component/toast/index'
import { kPasswordLen, kToastDurtion, kAccountNameRegex } from '../../config'
import EncryptHelper from '../../utils/EncryptHelper'
import RandomFactory from '../../utils/RandomFactory'
import DMCScatterInterface from '../../interface/Interface'
import { chain_host, version, chain_port, chain_chainId } from '../../config'
import PluginFactory from '../../plugins/PluginFactory'
import DMCScatter from '../../models/DMCScatter'
import { transfer } from '../../service/action'
import ConfirmModal from '../component/confirm-modal'
// @ts-ignore
import DMC from 'dmc.js'
import Identity from '../../models/Identity'
import Keypair from '../../models/Keypair'
import { amountRegex } from '../../utils/regex'
import reformChainError from '../../utils/reformError'
import { account } from '../../service/action'

interface ILanguageProps {
    match?: match
    history?: any
    dispatch?: Dispatch
    language: LanguageEnum
    password: string
    identitys: Array<Identity>
    keypairs: Array<Keypair>
    dmc: any
    buttonLoading: boolean
}

interface IHashProps {
    hash: string
}

class Extransfer extends Component<ILanguageProps> {
    private confirmModal: ConfirmModal
    private settingAction: typeof SettingActions
    private privateKey: string
    private dmc: any

    constructor(props: ILanguageProps) {
        super(props)
        const { match, identitys, keypairs } = props
        const { hash } = match.params as IHashProps
        this.settingAction = bindActionCreators(SettingActions, props.dispatch)
        const identity = identitys.find((identity) => identity.hash === hash)
        const keypair = keypairs.find(
            (keypair) => keypair.publicKey === identity.publicKey
        )
        this.privateKey = EncryptHelper.aesDecrypt(
            keypair.iv,
            RandomFactory.systemSalt(),
            keypair.privateKey
        )
        this.dmc = DMC({
            chainId: chain_chainId,
            keyProvider: this.privateKey,
            httpEndpoint: `http://${chain_host}:${chain_port}`,
            logger: {
                log: null,
                error: null,
            },
            keyPrefix: 'DM',
        })
    }

    state = {
        account: '',
        amount: '',
        memo: '',
        dmc_quantity: '',
        buttonLoading: false,
    }

    componentDidMount(): void {
        const { history } = this.props
        const name = history?.location?.state?.present_account
        ;(async () => {
            const asset_data = await account(name)
            const dmc_asset = asset_data?.rows?.filter((item: any) => {
                return item?.balance?.quantity?.split(' ')[1] === 'DMC'
            })[0]?.balance?.quantity
            this.setState({ dmc_quantity: dmc_asset || '0.0000' })
        })()
    }

    checkNumberPrecision = (number: string, precision: number) => {
        const numArr = number.split('.')
        if (numArr.length === 1) {
            return true
        }
        if (numArr.length === 2) {
            const decimal = numArr[1]
            if (decimal.length > precision) {
                return false
            }
            return true
        }
        return false
    }

    doneTransfer = async () => {
        Toast.loading('', 0)
        const { history } = this.props
        const { account, amount, memo } = this.state
        const actions = [
            {
                account: 'dmc.token',
                name: 'extransfer',
                authorization: [
                    {
                        actor: history?.location?.state?.present_account,
                        permission: 'active',
                    },
                ],
                data: {
                    from: history?.location?.state?.present_account,
                    to: account,
                    quantity: {
                        quantity: `${Number(amount).toFixed(4)} DMC`,
                        contract: 'datamall',
                    },
                    memo: memo,
                },
            },
        ]
        this.dmc
            .transact(
                { actions },
                {
                    blocksBehind: 3,
                    expireSeconds: 30,
                }
            )
            .then((res: any) => {
                Toast.success(
                    LocalizationHelper.valueForKey('succeed'),
                    kToastDurtion
                )
                this.componentDidMount()
                this.setState({
                    account: '',
                    amount: '',
                    memo: '',
                })
            })
            .catch((error: any) => {
                Toast.hide()
                reformChainError(
                    error,
                    history?.location?.state?.present_account
                )
            })
    }

    confirmTransfer = (evt: React.MouseEvent) => {
        const { account, memo, amount } = this.state
        if (account === '') {
            Toast.info(
                LocalizationHelper.valueForKey('receive_account_name'),
                kToastDurtion
            )
            return
        }
        // if (!kAccountNameRegex.test(account)) {
        //     Toast.info(
        //         LocalizationHelper.valueForKey('account_name_not_match'),
        //         kToastDurtion
        //     )
        //     return
        // }
        if (amount === '') {
            Toast.info(
                LocalizationHelper.valueForKey('transfer_amount_input'),
                kToastDurtion
            )
            return
        }
        if (!amountRegex.test(amount)) {
            Toast.info(
                LocalizationHelper.valueForKey('invaild_quality'),
                kToastDurtion
            )
            return
        }
        if (!this.checkNumberPrecision(amount, 4)) {
            Toast.info(
                LocalizationHelper.valueForKey('precision_error'),
                kToastDurtion
            )
            return
        }
        const target = evt.target as HTMLElement
        const offsetY = target.parentElement.parentElement.scrollTop
        let y: number = target.offsetTop
        this.confirmModal.show({
            y: 44 + 120,
        })
    }

    render() {
        const { history, password } = this.props
        const { dmc_quantity } = this.state
        return (
            <Father
                history={history}
                left={require('../../assets/back.png')}
                title={LocalizationHelper.valueForKey('extransfer')}
            >
                <div className='verify extransfer'>
                    <div className='token'>
                        <span>
                            {LocalizationHelper.valueForKey('type_currency')}
                        </span>
                        <input
                            className='textarea scroll input'
                            value={'DMC'}
                            disabled
                        />
                    </div>
                    <div className='account'>
                        <span>
                            {LocalizationHelper.valueForKey('receive_account')}
                        </span>
                        <input
                            title={LocalizationHelper.valueForKey(
                                'receive_account_name'
                            )}
                            className='textarea scroll input'
                            placeholder={LocalizationHelper.valueForKey(
                                'receive_account_name'
                            )}
                            maxLength={12}
                            value={this.state.account}
                            onChange={(e) =>
                                this.setState({
                                    account: e.target.value.trim(),
                                })
                            }
                        />
                    </div>
                    <div className='amount'>
                        <div className='amount_top'>
                            <span>
                                {LocalizationHelper.valueForKey(
                                    'transfer_amount'
                                )}
                            </span>
                            <div>
                                <span className='balance'>
                                    {LocalizationHelper.valueForKey(
                                        'myAssets_remain'
                                    )}
                                </span>
                                <span style={{ marginLeft: 5 }}>
                                    {dmc_quantity.split(' ')[0]}
                                </span>
                            </div>
                        </div>
                        <div className='amount_bottom'>
                            <input
                                title={LocalizationHelper.valueForKey(
                                    'transfer_amount_input'
                                )}
                                className='textarea scroll input'
                                placeholder={LocalizationHelper.valueForKey(
                                    'transfer_amount_input'
                                )}
                                value={this.state.amount}
                                onChange={(e) =>
                                    this.setState({
                                        amount: e.target.value.trim(),
                                    })
                                }
                            />
                            <div
                                className='transfer_all'
                                onClick={() => {
                                    this.setState({
                                        amount: dmc_quantity.split(' ')[0],
                                    })
                                }}
                            >
                                {LocalizationHelper.valueForKey('all_out')}
                            </div>
                        </div>
                    </div>
                    <div className='memo'>
                        <span>
                            {LocalizationHelper.valueForKey('transfer_memo')}
                        </span>
                        <input
                            className='textarea scroll input'
                            placeholder={LocalizationHelper.valueForKey(
                                'transfer_memo'
                            )}
                            // maxLength={50}
                            value={this.state.memo}
                            onChange={(e) =>
                                this.setState({
                                    memo: e.target.value,
                                })
                            }
                        />
                    </div>
                    <div
                        className='btn'
                        onClick={(evt) => {
                            this.confirmTransfer(evt)
                        }}
                    >
                        {LocalizationHelper.valueForKey('transfer')}
                    </div>
                </div>
                <ConfirmModal
                    ref={(e) => (this.confirmModal = e)}
                    callback={this.doneTransfer}
                    password={password}
                ></ConfirmModal>
            </Father>
        )
    }
}

function mapStateToProps({ setting, keychain }: IStoreState) {
    return {
        language: setting.language,
        password: setting.password,
        keypairs: keychain.keypairs,
        identitys: keychain.identitys,
    }
}

export default connect(mapStateToProps)(Extransfer)
