import React, { Component } from 'react'
import { connect } from 'react-redux'
import { bindActionCreators, Dispatch } from 'redux'
import { History } from 'history'
import { IStoreState } from '../../reducer/index'
import Identity from '../../models/Identity'
import LocalizationHelper from '../../localization/LocalizationHelper'
import Pop2 from '../component/pop2'
import Pop2Setting from '../component/pop2'
import Network from '../../models/Network'
import * as KeyChainActions from '../../actions/keychain'
import DMCScatterInterface from '../../interface/Interface'
import { kToastDurtion, version } from '../../config'
import Toast from '../component/toast/index'
import Modal from '../component/modal/index'
import { getVersion } from '../../service/request'

interface IMainProps {
    dispatch?: Dispatch
    history?: History
    identitys: Array<Identity>
    networks: Array<Network>
    created: boolean
}

interface IMainState {}

class Main extends Component<IMainProps, IMainState> {
    private pop: Pop2
    private popSetting: Pop2Setting
    private keyChainAction: typeof KeyChainActions

    constructor(props: IMainProps) {
        super(props)
        this.keyChainAction = bindActionCreators(
            KeyChainActions,
            props.dispatch
        )
    }

    state = {
        selected_name: '',
        updated_version: '',
        is_updated_version: true,
        update_url: '',
    }

    componentDidMount(): void {
        ;(async () => {
            const versionInfo = await getVersion()
            this.setState({
                // is_updated_version: true,
                is_updated_version: !this.Version(
                    versionInfo.data.version,
                    version
                ),
                // is_updated_version: versionInfo.data.version === version,
                updated_version: versionInfo.data.version,
                update_url: versionInfo.data.url,
            })
        })()
    }

    mouseEnter(evt: React.MouseEvent, hash: string, name: string) {
        const target = evt.target as HTMLElement
        const isChild = evt.target !== evt.currentTarget
        const offsetY = isChild
            ? target.parentElement.parentElement.parentElement.scrollTop
            : target.parentElement.parentElement.scrollTop
        let y: number = isChild
            ? target.parentElement.offsetTop
            : target.offsetTop
        this.setState({ selected_name: name })
        this.pop.show({
            r: 18,
            y: y + 44 - offsetY,
            hash,
            options: [
                // {
                //     icon: require('../../assets/whitelist.png'),
                //     title: LocalizationHelper.valueForKey('whitelist_manager'),
                // },
                {
                    icon: require('../../assets/assets.png'),
                    title: LocalizationHelper.valueForKey('check_assets'),
                },
                {
                    icon: require('../../assets/export.png'),
                    title: LocalizationHelper.valueForKey('export_privatekey'),
                },
                {
                    icon: require('../../assets/delete.png'),
                    title: LocalizationHelper.valueForKey('delete_account'),
                },
            ],
        })
    }

    mouseEnterSetting(evt: React.MouseEvent) {
        const { updated_version, is_updated_version } = this.state
        const target = evt.target as HTMLElement
        const offsetY = target.parentElement.parentElement.scrollTop
        const isChild = evt.target !== evt.currentTarget
        let y: number = isChild
            ? target.parentElement.offsetTop
            : target.offsetTop
        this.popSetting.show({
            r: 10,
            y: y + 44 - offsetY,
            options: is_updated_version
                ? [
                      {
                          title: LocalizationHelper.valueForKey('language'),
                      },
                      {
                          title: LocalizationHelper.valueForKey(
                              'chain_manager'
                          ),
                      },
                      {
                          title: LocalizationHelper.valueForKey(
                              'change_password'
                          ),
                      },
                  ]
                : [
                      {
                          title: LocalizationHelper.valueForKey('language'),
                      },
                      {
                          title: LocalizationHelper.valueForKey(
                              'chain_manager'
                          ),
                      },
                      {
                          title: LocalizationHelper.valueForKey(
                              'change_password'
                          ),
                      },
                      {
                          ...(!is_updated_version && {
                              title: `${LocalizationHelper.valueForKey(
                                  'version_update'
                              )} V${updated_version} ${
                                  is_updated_version ? '' : '*'
                              }`,
                          }),
                      },
                  ],
        })
    }

    onSelect(idx: number, hash?: string) {
        const { history } = this.props
        switch (idx) {
            // case 0:
            //     history.push(`/accountmanager/${hash}`)
            //     break
            case 0:
                history.push(`/assets/${hash}`, {
                    name: this.state.selected_name,
                })
                break
            case 1:
                history.push(`/exportpk/${hash}`)
                break
            default:
                this.confirmDelete(hash)
                break
        }
    }

    onSettingSelect(idx: number) {
        const { history } = this.props
        switch (idx) {
            case 0:
                history.push(`/language`)
                break
            case 1:
                history.push(`/chain`)
                break
            case 2:
                history.push(`/change-password`)
                break
            case 3:
                window.open(this.state.update_url, '_blank')
                break
            default:
                break
        }
    }

    format(str: string, arg: string): string {
        return str.replace(new RegExp('\\{0\\}', 'g'), arg)
    }

    Version(curV: string, reqV: string) {
        let arr1 = curV.toString().split('.')
        let arr2 = reqV.toString().split('.')
        //将两个版本号拆成数字
        let minL = Math.min(arr1.length, arr2.length)
        let pos = 0 //当前比较位
        let diff = 0 //当前为位比较是否相等
        let flag = false
        //逐个比较如果当前位相等则继续比较下一位
        while (pos < minL) {
            diff = parseInt(arr1[pos]) - parseInt(arr2[pos])
            if (diff == 0) {
                pos++
                continue
            } else if (diff > 0) {
                flag = true
                break
            } else {
                flag = false
                break
            }
        }
        return flag
    }
    /**
     * 确认是否删除账户
     * @param hash hash
     */
    confirmDelete(hash?: string) {
        const { identitys } = this.props
        const identity = identitys.find((identity) => identity.hash === hash)
        const deleteList = identitys.filter((id) => {
            return id.name.split('@')[0] === identity.name.split('@')[0]
        })
        if (!identity) {
            return
        }
        Modal.alert(
            <span className='alert_title'>
                {LocalizationHelper.valueForKey('delete_identity')}
            </span>,
            <span className='alert_message'>
                {this.format(
                    LocalizationHelper.valueForKey(
                        'unable_transaction_after_delete'
                    ),
                    identity.name.split('@')[0]
                )}
            </span>,
            [
                {
                    text: LocalizationHelper.valueForKey('cancel'),
                    style: {
                        color: '#333333',
                        fontSize: '14px',
                        cursor: 'pointer',
                    },
                },
                {
                    text: LocalizationHelper.valueForKey('delete'),
                    style: {
                        color: 'red',
                        fontSize: '14px',
                        cursor: 'pointer',
                    },
                    onPress: () => {
                        deleteList.map((identity) => {
                            this.deleteAccount(identity)
                        })
                    },
                },
            ]
        )
    }

    /**
     * 删除账户
     * @param identity
     */
    deleteAccount(identity: Identity) {
        DMCScatterInterface.delIdentity(identity)
        this.keyChainAction.delIdentity(identity)
        Toast.success(
            LocalizationHelper.valueForKey('delete_account_success'),
            kToastDurtion
        )
    }

    renderNavBar() {
        const { is_updated_version } = this.state
        return (
            <div className='navbar'>
                <span className='title main_title'>
                    {LocalizationHelper.valueForKey('account_list')}
                </span>
                <div className='menu_icon'>
                    <span className='right'>
                        <img
                            className='back_icon'
                            src={require('../../assets/setting.png')}
                            onClick={(evt) => this.mouseEnterSetting(evt)}
                        />
                        {is_updated_version ? (
                            ''
                        ) : (
                            <div style={{ color: 'red' }}>*</div>
                        )}
                    </span>
                    {/* <span className='right'>
                        <img className='back_icon' src={require('../../assets/chain.png')} />
                    </span> */}
                </div>
            </div>
        )
    }

    renderNoAccount() {
        const { created } = this.props
        return (
            <div className='content scroll'>
                <div className='no_account'>
                    {LocalizationHelper.valueForKey('no_account')}
                </div>
            </div>
        )
    }

    renderAccounts() {
        const { identitys, networks, created } = this.props
        const dict: { [key: string]: string } = {}
        networks.forEach((network) => (dict[network.unique2()] = network.name))
        const activeList = identitys.filter(
            (item) => item.name.split('@')[1] === 'active'
        )
        const lastList = identitys.filter(
            (item) =>
                !activeList
                    .map((item) => item.name.split('@')[0])
                    .includes(item.name.split('@')[0])
        )
        const obj: { [key: string]: boolean } = {}
        const listData = lastList.reduce((item, next) => {
            if (!obj[next.name.split('@')[0]]) {
                item.push(next)
                obj[next.name.split('@')[0]] = true
            }
            return item
        }, [])
        const finalList = activeList.concat(listData)
        return (
            <div className='content scroll accounts'>
                {finalList.map((identity) => (
                    <div
                        className='account_item account_item2'
                        key={identity.hash}
                    >
                        <div className='preview'>
                            <span className='account_name'>
                                {identity.name &&
                                    identity.name.length > 0 &&
                                    identity.name.split('@')[0]}
                            </span>
                            <span className='account_name'>
                                {dict[Object.keys(identity.accounts)[0]]}
                            </span>
                        </div>
                        <span
                            className='account_setting_mask'
                            onClick={(evt) =>
                                this.mouseEnter(
                                    evt,
                                    identity.hash,
                                    identity.name.split('@')[0]
                                )
                            }
                        >
                            <img
                                className='account_setting'
                                src={require('../../assets/account_setting.png')}
                            />
                        </span>
                    </div>
                ))}
            </div>
        )
    }

    renderFooter() {
        const { history, created, identitys } = this.props
        const disabled = identitys.length === 0 && created
        return (
            <div className='footer'>
                <div
                    className={`create ${disabled ? 'disabled' : ''}`}
                    onClick={() => {
                        if (disabled) return
                        history.push('/create')
                    }}
                >
                    {LocalizationHelper.valueForKey('create_accout')}
                </div>
                <div className='import' onClick={() => history.push('/import')}>
                    {LocalizationHelper.valueForKey('import_account')}
                </div>
            </div>
        )
    }

    render() {
        const { identitys } = this.props
        return (
            <div className='main'>
                {this.renderNavBar()}
                {identitys.length > 0
                    ? this.renderAccounts()
                    : this.renderNoAccount()}
                {this.renderFooter()}
                <Pop2
                    ref={(e) => (this.pop = e)}
                    callback={(idx, hash) => this.onSelect(idx, hash)}
                />
                <Pop2Setting
                    ref={(e) => (this.popSetting = e)}
                    callback={(idx) => this.onSettingSelect(idx)}
                />
            </div>
        )
    }
}

function mapStateToProps({ keychain, setting }: IStoreState) {
    return {
        identitys: keychain.identitys,
        networks: setting.networks,
        created: setting.created,
    }
}

export default connect(mapStateToProps)(Main)
