import React, { Component } from 'react'
import { bindActionCreators, Dispatch } from 'redux'
import { format } from 'date-fns'
import { match } from 'react-router-dom'
import { History } from 'history'
import { connect } from 'react-redux'
import { IStoreState } from '../../reducer/index'
import Identity from '../../models/Identity'
import Permission from '../../models/Permission'
import * as KeyChainActions from '../../actions/keychain'
import LocalizationHelper from '../../localization/LocalizationHelper'
import Father from '../component/father'
import DMCScatterInterface from '../../interface/Interface'
import Modal from '../component/modal/index'
import UtilsHelper from '../../utils/UtilsHelper'
import GroupHelper from '../../utils/GroupHelper'
import ReformTokenName from '../../utils/ReformTokenName'
import { account, lockAccount } from '../../service/action'
import { findLockRecord } from '../../service/request'
import Toast from '../component/toast/index'
import BigNumber from 'bignumber.js'
import reformChainError from '../../utils/reformError'

interface IAccountManagerProps {
    identitys: Array<Identity>
    match?: match
    permissions: Array<Permission>
    history?: any
    dispatch?: Dispatch
}

interface IHashProps {
    hash: string
}

class Assets extends Component<IAccountManagerProps> {
    private keyChainAction: typeof KeyChainActions

    constructor(props: IAccountManagerProps) {
        super(props)
        this.keyChainAction = bindActionCreators(
            KeyChainActions,
            props.dispatch
        )
    }

    state = {
        showWhichTab: 'assets',
        lock_list: [] as any,
        lock_record: [] as any,
        locked_quantity: '',
    }

    componentDidMount(): void {
        ;(async () => {
            try {
                Toast.loading('', 0)
                const { history } = this.props
                const name = history.location.state.present_account
                const lock_data = await lockAccount(name)
                const lock_asset = lock_data?.rows?.filter((item: any) => {
                    return item?.balance?.quantity?.split(' ')[1] === 'DMC'
                })
                this.setState({ lock_list: lock_asset })
                const dmcSum = lock_asset.reduce((pre: any, cur: any) => {
                    return new BigNumber(pre)
                        .plus(new BigNumber(cur.balance.quantity.split(' ')[0]))
                        .toFixed(4)
                }, 0)
                this.setState({
                    locked_quantity: new BigNumber(dmcSum).toFixed(4),
                })
                const lock_record = await findLockRecord(name)
                const record = lock_record?.data?.data?.find_tokens_action
                const record_list = record.map((item: any) => {
                    if (item.contract_action === 'dmc.token/redeemrec') {
                        return {
                            date: format(
                                new Date(
                                    `${item?.action?.rawData?.block_time}Z`
                                ),
                                'yyyy-MM-dd HH:mm:ss'
                            ),
                            quantity: `+${
                                item?.action?.rawData?.act?.data?.asset?.quantity.split(
                                    ' '
                                )[0]
                            }`,
                            action: 'lock',
                        }
                    } else if (item.contract_action === 'dmc.token/exunlock') {
                        return {
                            date: format(
                                new Date(
                                    `${item?.action?.rawData?.block_time}Z`
                                ),
                                'yyyy-MM-dd HH:mm:ss'
                            ),
                            quantity: `-${
                                item?.action?.rawData?.act?.data?.quantity?.quantity?.split(
                                    ' '
                                )[0]
                            }`,
                            action: 'unlock',
                        }
                    } else if (
                        item.contract_action === 'dmc.token/exlocktrans'
                    ) {
                        return {
                            date: format(
                                new Date(
                                    `${item?.action?.rawData?.block_time}Z`
                                ),
                                'yyyy-MM-dd HH:mm:ss'
                            ),
                            quantity: `+${
                                item?.action?.rawData?.act?.data?.quantity?.quantity?.split(
                                    ' '
                                )[0]
                            }`,
                            action: 'lock',
                        }
                    } else if (item.contract_action === 'dmc.token/exlock') {
                        return {
                            date: format(
                                new Date(
                                    `${item?.action?.rawData?.block_time}Z`
                                ),
                                'yyyy-MM-dd HH:mm:ss'
                            ),
                            quantity: `+${
                                item?.action?.rawData?.act?.data?.quantity?.quantity?.split(
                                    ' '
                                )[0]
                            }`,
                            action: 'lock',
                        }
                    }
                })
                this.setState({ lock_record: record_list })
                Toast.hide()
            } catch (error) {
                reformChainError(error)
            }
        })()
    }

    render() {
        const { lock_list, lock_record, locked_quantity } = this.state
        const { history } = this.props
        const name = history.location.state.present_account
        return (
            <Father
                history={history}
                left={require('../../assets/back.png')}
                title={LocalizationHelper.valueForKey('assets')}
            >
                <div className='lockInfo'>
                    <div>{name}</div>
                    <div className='assetsAmount'>
                        <div>
                            DMC{' '}
                            {LocalizationHelper.valueForKey('lock_token_hint')}
                        </div>
                        <div>{locked_quantity}</div>
                    </div>
                    <div className='bottom_tab'>
                        <div className='bottom_title'>
                            {LocalizationHelper.valueForKey('locked_amount')}
                        </div>
                        {lock_list.map((item: any) => {
                            return (
                                <div
                                    className='bottom_detail'
                                    onClick={() => {
                                        const hash =
                                            history?.location?.state?.hash
                                        history.push('/unlock', {
                                            locked_amount:
                                                item?.balance?.quantity?.split(
                                                    ' '
                                                )[0],
                                            locked_time: item?.lock_timestamp,
                                            hash: hash,
                                            present_account: name,
                                        })
                                    }}
                                >
                                    <div className='detail_amount'>
                                        <span>DMC</span>
                                        <span>
                                            {
                                                item?.balance?.quantity?.split(
                                                    ' '
                                                )[0]
                                            }
                                        </span>
                                    </div>
                                    <div className='detail_time'>
                                        {`${LocalizationHelper.valueForKey(
                                            'lock_to'
                                        )}
                                        ${format(
                                            new Date(`${item.lock_timestamp}Z`),
                                            'yyyy-MM-dd HH:mm:ss'
                                        )}`}
                                    </div>
                                </div>
                            )
                        })}
                    </div>
                </div>
                <div className='lockList'>
                    <div className='tabs'>
                        <span className='showTab'>
                            {LocalizationHelper.valueForKey('recent_trx')}
                        </span>
                    </div>
                    {lock_record.map((item: any) => {
                        return (
                            <div className='trx_detail'>
                                <div className='detail_top'>
                                    <div className='title'>
                                        <img
                                            width={16}
                                            src={
                                                item?.action === 'unlock'
                                                    ? require('../../assets/unlock.png')
                                                    : require('../../assets/lock.png')
                                            }
                                        ></img>
                                        <span>
                                            {item.action === 'unlock'
                                                ? LocalizationHelper.valueForKey(
                                                      'ex_unlock'
                                                  )
                                                : LocalizationHelper.valueForKey(
                                                      'lock_token_hint'
                                                  )}
                                        </span>
                                    </div>
                                    <div>{item?.quantity}</div>
                                </div>
                                <div className='detail_bottom'>
                                    <div className='time'>{item?.date}</div>
                                    <div>
                                        {LocalizationHelper.valueForKey(
                                            'confirmed'
                                        )}
                                    </div>
                                </div>
                            </div>
                        )
                    })}
                </div>
            </Father>
        )
    }
}

function mapStateToProps({ keychain }: IStoreState) {
    return {
        identitys: keychain.identitys,
        permissions: keychain.permissions,
    }
}

export default connect(mapStateToProps)(Assets)
