import React, { Component } from 'react'
import { connect } from 'react-redux'
import { bindActionCreators, Dispatch } from 'redux'
import { match } from 'react-router-dom'
import Father from '../component/father'
import { History } from 'history'
import LocalizationHelper from '../../localization/LocalizationHelper'
import { IStoreState } from '../../reducer'
import * as SettingActions from '../../actions/setting'
import { LanguageEnum } from '../../models/Enum'
import Toast from '../component/toast/index'
import { kPasswordLen, kToastDurtion } from '../../config'
import EncryptHelper from '../../utils/EncryptHelper'
import RandomFactory from '../../utils/RandomFactory'
import DMCScatterInterface from '../../interface/Interface'
import { chain_host, version, dmc_website } from '../../config'
import PluginFactory from '../../plugins/PluginFactory'
import DMCScatter from '../../models/DMCScatter'
import QRCode from 'qrcode.react'

interface ILanguageProps {
    match?: match
    history?: any
    dispatch?: Dispatch
    language: LanguageEnum
    password: string
}

class ChangePassword extends Component<ILanguageProps> {
    private settingAction: typeof SettingActions

    constructor(props: ILanguageProps) {
        super(props)
        this.settingAction = bindActionCreators(SettingActions, props.dispatch)
    }

    state = {
        amount: '',
        memo: '',
        type: '',
        contract: '',
    }

    checkPwd(pwd: string): boolean {
        const { password } = this.props
        if (pwd.length <= 0) {
            Toast.info(
                LocalizationHelper.valueForKey('password_empty'),
                kToastDurtion
            )
            return false
        }
        const hashed = EncryptHelper.md5(
            EncryptHelper.md5(pwd),
            RandomFactory.systemSalt()
        )
        if (hashed !== password) {
            Toast.info(
                LocalizationHelper.valueForKey('password_error'),
                kToastDurtion
            )
            return false
        }
        return true
    }

    render() {
        const { history } = this.props
        const { type, contract, amount, memo } = this.state
        const account = history?.location?.state?.present_account
        // const current = walletManager.getCurrentAccount()
        const collectionAddress = `${dmc_website}/Pay?params=${account},DMC,datamall,,`
        return (
            <Father
                history={history}
                left={require('../../assets/back.png')}
                title={LocalizationHelper.valueForKey('currency_collection')}
            >
                <div className='receive_container'>
                    <div className='title'>
                        {LocalizationHelper.valueForKey('scan_to_pay')}
                    </div>
                    <div className='qr'>
                        <QRCode
                            id='receive'
                            value={collectionAddress}
                            size={180}
                            bgColor='white'
                            fgColor='black'
                        />
                    </div>
                </div>
            </Father>
        )
    }
}

function mapStateToProps({ setting }: IStoreState) {
    return {
        language: setting.language,
        password: setting.password,
    }
}

export default connect(mapStateToProps)(ChangePassword)
